#!/usr/bin/env node

// src/index.ts
import {
  intro,
  outro,
  log as log4,
  text,
  confirm as confirm2,
  group as group2,
  spinner as spinner2
} from "@clack/prompts";
import color from "picocolors";
import { TERMINAL_INFO } from "@ionic/utils-terminal";
import { Project as Project3 } from "ts-morph";
import { existsSync as existsSync2 } from "node:fs";
import { cwd } from "node:process";

// src/angular/migrations/standalone/0001-migrate-app-module.ts
import { SyntaxKind, ts } from "ts-morph";

// src/angular/utils/log-utils.ts
import { log } from "@clack/prompts";
async function saveFileChanges(sourceFile, cliOptions) {
  sourceFile.formatText();
  if (cliOptions.dryRun) {
    log.info("[Dry Run] Writing changes to: " + sourceFile.getFilePath());
    log.info(sourceFile.getFullText());
  } else {
    await sourceFile.save();
  }
  return sourceFile.getFullText();
}

// src/angular/migrations/standalone/0001-migrate-app-module.ts
var migrateAppModule = async (project, cliOptions) => {
  var _a, _b;
  const appModule = project.getSourceFile("app.module.ts");
  if (appModule === void 0) {
    return;
  }
  const importDeclaration = appModule.getImportDeclaration("@ionic/angular");
  if (!importDeclaration) {
    return;
  }
  importDeclaration.setModuleSpecifier("@ionic/angular/standalone");
  const namedImports = importDeclaration.getNamedImports();
  const importSpecifier = namedImports.find(
    (n) => n.getName() === "IonicModule"
  );
  if (importSpecifier) {
    importSpecifier.remove();
  }
  importDeclaration.addNamedImport("provideIonicAngular");
  const ngModuleDecorator = (_a = appModule.getClass("AppModule")) == null ? void 0 : _a.getDecorator("NgModule");
  if (!ngModuleDecorator) {
    return;
  }
  const ngModuleDecoratorArguments = ngModuleDecorator.getArguments()[0];
  const importsProperty = ngModuleDecoratorArguments.getDescendantsOfKind(SyntaxKind.PropertyAssignment).find((n) => n.compilerNode.name.getText() === "imports");
  let ionicModuleForRootCallExpression;
  if (importsProperty) {
    const importsPropertyAssignment = importsProperty.getInitializerIfKind(
      SyntaxKind.ArrayLiteralExpression
    );
    if (importsPropertyAssignment) {
      const ionicModule = importsPropertyAssignment.getElements().find((e) => {
        if (ts.isCallExpression(e.compilerNode)) {
          const identifier = e.compilerNode.expression;
          if (identifier.getText() === "IonicModule.forRoot") {
            return true;
          }
        }
        return false;
      });
      if (ionicModule) {
        ionicModuleForRootCallExpression = ionicModule.compilerNode;
        importsPropertyAssignment.removeElement(ionicModule);
      }
    }
  }
  let providersProperty = ngModuleDecoratorArguments.getDescendantsOfKind(SyntaxKind.PropertyAssignment).find((n) => n.compilerNode.name.getText() === "providers");
  if (!providersProperty) {
    providersProperty = ngModuleDecoratorArguments.asKind(SyntaxKind.ObjectLiteralExpression).addPropertyAssignment({
      name: "providers",
      initializer: "[]"
    });
  }
  const providersPropertyAssignment = providersProperty.getInitializerIfKind(
    SyntaxKind.ArrayLiteralExpression
  );
  providersPropertyAssignment == null ? void 0 : providersPropertyAssignment.addElement(
    `provideIonicAngular(${(_b = ionicModuleForRootCallExpression == null ? void 0 : ionicModuleForRootCallExpression.arguments) == null ? void 0 : _b.map((a) => a.getText()).join(", ")})`
  );
  return await saveFileChanges(appModule, cliOptions);
};

// src/angular/utils/angular-utils.ts
import { SyntaxKind as SyntaxKind3 } from "ts-morph";

// src/angular/utils/decorator-utils.ts
import { SyntaxKind as SyntaxKind2 } from "ts-morph";
var getDecoratorArgument = (decorator, propertyName) => {
  const args = decorator.getArguments();
  if (args.length === 0) {
    return;
  }
  const arg = args[0];
  const prop = arg.getDescendantsOfKind(SyntaxKind2.PropertyAssignment).find((n) => n.compilerNode.name.getText() === propertyName);
  return prop;
};
var insertIntoDecoratorArgArray = (decorator, propertyName, value) => {
  let property = getDecoratorArgument(decorator, propertyName);
  if (!property) {
    property = decorator.getArguments()[0].asKind(SyntaxKind2.ObjectLiteralExpression).addPropertyAssignment({
      name: propertyName,
      initializer: "[]"
    });
  }
  const propertyInitializer = property.getInitializerIfKind(
    SyntaxKind2.ArrayLiteralExpression
  );
  propertyInitializer.addElement(value);
};
var deleteFromDecoratorArgArray = (decorator, propertyName, value) => {
  const property = getDecoratorArgument(decorator, propertyName);
  if (!property) {
    return;
  }
  const propertyInitializer = property.getInitializerIfKind(
    SyntaxKind2.ArrayLiteralExpression
  );
  const element = propertyInitializer.getElements().find((e) => e.getText() === value);
  if (element) {
    propertyInitializer.removeElement(element);
  }
};

// src/angular/utils/angular-utils.ts
function findNgModuleClassForComponent(sourceFile) {
  var _a;
  const componentClassName = (_a = sourceFile.getClasses()[0]) == null ? void 0 : _a.getName();
  if (!componentClassName) {
    return;
  }
  const sourceFiles = sourceFile.getProject().getSourceFiles();
  const ngModuleClass = sourceFiles.find((f) => {
    var _a2;
    const ngModuleDecorator = (_a2 = f.getClasses()[0]) == null ? void 0 : _a2.getDecorator("NgModule");
    if (!ngModuleDecorator) {
      return false;
    }
    const declarationsProperty = getDecoratorArgument(
      ngModuleDecorator,
      "declarations"
    );
    if (!declarationsProperty) {
      return false;
    }
    const declarationsArray = declarationsProperty.getInitializerIfKind(
      SyntaxKind3.ArrayLiteralExpression
    );
    if (!declarationsArray) {
      return false;
    }
    const componentClass = declarationsArray.getElements().find((e) => {
      const identifier = e.compilerNode;
      return identifier.getText() === componentClassName;
    });
    if (!componentClass) {
      return false;
    }
    return true;
  });
  return ngModuleClass;
}
function findComponentTypescriptFileForTemplateFile(templateFile) {
  const templateFilePath = templateFile.getFilePath();
  const templateFileName = templateFile.getBaseNameWithoutExtension();
  const sourceFiles = templateFile.getProject().getSourceFiles();
  const componentTypescriptFile = sourceFiles.find((f) => {
    const filePath = f.getFilePath();
    const fileName = f.getBaseNameWithoutExtension();
    if (filePath === templateFilePath) {
      return false;
    }
    if (fileName !== templateFileName) {
      return false;
    }
    if (!isAngularComponentClass(f)) {
      return false;
    }
    return true;
  });
  return componentTypescriptFile;
}
function addImportToComponentDecorator(sourceFile, importName) {
  if (!isAngularComponentStandalone(sourceFile)) {
    console.warn(
      "[Ionic Dev] Cannot add import to component decorator. Component is not standalone."
    );
    return;
  }
  const componentDecorator = getAngularComponentDecorator(sourceFile);
  insertIntoDecoratorArgArray(componentDecorator, "imports", importName);
  sourceFile.formatText();
}
function removeImportFromComponentDecorator(sourceFile, importName) {
  if (!isAngularComponentStandalone(sourceFile)) {
    console.warn(
      "[Ionic Dev] Cannot remove import from component decorator. Component is not standalone."
    );
    return;
  }
  const componentDecorator = getAngularComponentDecorator(sourceFile);
  deleteFromDecoratorArgArray(componentDecorator, "imports", importName);
  sourceFile.formatText();
}
var addImportToNgModuleDecorator = (sourceFile, importName) => {
  const ngModuleDecorator = getAngularNgModuleDecorator(sourceFile);
  if (ngModuleDecorator) {
    insertIntoDecoratorArgArray(ngModuleDecorator, "imports", importName);
    sourceFile.formatText();
  }
};
var removeImportFromNgModuleDecorator = (sourceFile, importName) => {
  const ngModuleDecorator = getAngularNgModuleDecorator(sourceFile);
  if (ngModuleDecorator) {
    deleteFromDecoratorArgArray(ngModuleDecorator, "imports", importName);
  }
};
function isAngularComponentStandalone(sourceFile) {
  if (!isAngularComponentClass(sourceFile)) {
    return false;
  }
  const componentDecorator = getAngularComponentDecorator(sourceFile);
  if (!componentDecorator) {
    return false;
  }
  const standalonePropertyAssignment = getDecoratorArgument(
    componentDecorator,
    "standalone"
  );
  if (!standalonePropertyAssignment) {
    return false;
  }
  const standalonePropertyValue = standalonePropertyAssignment.getInitializerIfKind(SyntaxKind3.TrueKeyword);
  if (!standalonePropertyValue) {
    return false;
  }
  return true;
}
function isAngularComponentClass(sourceFile) {
  const componentDecorator = getAngularComponentDecorator(sourceFile);
  if (!componentDecorator) {
    return false;
  }
  const importDeclaration = sourceFile.getImportDeclaration("@angular/core");
  if (!importDeclaration) {
    return false;
  }
  const namedImports = importDeclaration.getNamedImports();
  const componentImportSpecifier = namedImports.find(
    (n) => n.getName() === "Component"
  );
  if (!componentImportSpecifier) {
    return false;
  }
  return true;
}
function getAngularComponentDecorator(sourceFile) {
  var _a;
  const componentDecorator = (_a = sourceFile.getClasses()[0]) == null ? void 0 : _a.getDecorator("Component");
  return componentDecorator;
}
function getAngularNgModuleDecorator(sourceFile) {
  var _a;
  const ngModuleDecorator = (_a = sourceFile.getClasses()[0]) == null ? void 0 : _a.getDecorator("NgModule");
  return ngModuleDecorator;
}

// src/angular/migrations/standalone/0004-migrate-import-statements.ts
var migrateImportStatements = async (project, cliOptions) => {
  for (const sourceFile of project.getSourceFiles()) {
    let hasChanges = false;
    const importDeclarations = sourceFile.getImportDeclarations();
    importDeclarations.forEach((importDeclaration) => {
      const moduleSpecifier = importDeclaration.getModuleSpecifierValue();
      if (moduleSpecifier === "@ionic/angular") {
        importDeclaration.setModuleSpecifier("@ionic/angular/standalone");
        const namedImports = importDeclaration.getNamedImports();
        const importSpecifier = namedImports.find(
          (n) => n.getName() === "IonicModule"
        );
        if (importSpecifier) {
          if (namedImports.length > 1) {
            importSpecifier.remove();
          } else {
            importDeclaration.remove();
          }
          removeImportFromNgModuleDecorator(sourceFile, "IonicModule");
        }
        hasChanges = true;
      }
    });
    if (hasChanges) {
      await saveFileChanges(sourceFile, cliOptions);
    }
  }
};

// src/angular/migrations/standalone/0002-import-standalone-component.ts
import { SyntaxKind as SyntaxKind4 } from "ts-morph";
import { parse } from "@angular-eslint/template-parser";
import { log as log2 } from "@clack/prompts";

// src/angular/utils/ionic-utils.ts
var IONIC_COMPONENTS = [
  "ion-app",
  "ion-action-sheet",
  "ion-alert",
  "ion-accordion",
  "ion-accordion-group",
  "ion-avatar",
  "ion-backdrop",
  "ion-back-button",
  "ion-badge",
  "ion-breadcrumb",
  "ion-breadcrumbs",
  "ion-button",
  "ion-buttons",
  "ion-card",
  "ion-card-content",
  "ion-card-header",
  "ion-card-subtitle",
  "ion-card-title",
  "ion-checkbox",
  "ion-chip",
  "ion-col",
  "ion-content",
  "ion-datetime",
  "ion-datetime-button",
  "ion-fab",
  "ion-fab-button",
  "ion-fab-list",
  "ion-footer",
  "ion-grid",
  "ion-header",
  "ion-icon",
  "ion-img",
  "ion-infinite-scroll",
  "ion-infinite-scroll-content",
  "ion-input",
  "ion-item",
  "ion-item-divider",
  "ion-item-group",
  "ion-item-sliding",
  "ion-item-options",
  "ion-item-option",
  "ion-label",
  "ion-list",
  "ion-list-header",
  "ion-loading",
  "ion-menu",
  "ion-menu-button",
  "ion-menu-toggle",
  "ion-modal",
  "ion-nav",
  "ion-nav-link",
  "ion-note",
  "ion-picker",
  "ion-popover",
  "ion-progress-bar",
  "ion-radio",
  "ion-radio-group",
  "ion-range",
  "ion-refresher",
  "ion-refresher-content",
  "ion-reorder",
  "ion-reorder-group",
  "ion-ripple-effect",
  "ion-router",
  "ion-router-link",
  "ion-router-outlet",
  "ion-route",
  "ion-route-redirect",
  "ion-row",
  "ion-searchbar",
  "ion-segment",
  "ion-segment-button",
  "ion-select",
  "ion-select-option",
  "ion-skeleton-text",
  "ion-spinner",
  "ion-split-pane",
  "ion-tab",
  "ion-tabs",
  "ion-tab-bar",
  "ion-tab-button",
  "ion-text",
  "ion-textarea",
  "ion-thumbnail",
  "ion-toolbar",
  "ion-toast",
  "ion-toggle",
  "ion-title"
];

// src/angular/utils/string-utils.ts
function kebabCaseToPascalCase(str) {
  return str.split("-").map((segment) => segment.charAt(0).toUpperCase() + segment.slice(1)).join("");
}
function kebabCaseToCamelCase(str) {
  return str.split("-").map((segment, index) => {
    if (index === 0) {
      return segment;
    }
    return segment.charAt(0).toUpperCase() + segment.slice(1);
  }).join("");
}

// src/angular/utils/typescript-utils.ts
function getOrCreateConstructor(sourceFile) {
  let constructor = sourceFile.getClasses()[0].getConstructors()[0];
  if (!constructor) {
    constructor = sourceFile.getClasses()[0].addConstructor();
  }
  return constructor;
}
function addImportToClass(sourceFile, importName, moduleSpecifier) {
  const addImport = (sourceFile2, importName2, moduleSpecifier2) => {
    const classDeclaration = sourceFile2.getClasses()[0];
    if (!classDeclaration) {
      return;
    }
    let importDeclaration = sourceFile2.getImportDeclaration(moduleSpecifier2);
    if (!importDeclaration) {
      importDeclaration = sourceFile2.addImportDeclaration({
        moduleSpecifier: moduleSpecifier2
      });
    }
    const importSpecifier = importDeclaration.getNamedImports().find((n) => n.getName() === importName2);
    if (!importSpecifier) {
      importDeclaration.addNamedImport(importName2);
    }
  };
  if (Array.isArray(importName)) {
    importName.forEach((name) => {
      addImport(sourceFile, name, moduleSpecifier);
    });
  } else {
    addImport(sourceFile, importName, moduleSpecifier);
  }
}
function removeImportFromClass(sourceFile, importName, moduleSpecifier) {
  const removeImport = (sourceFile2, importName2, moduleSpecifier2) => {
    const importDeclaration = sourceFile2.getImportDeclaration(moduleSpecifier2);
    if (!importDeclaration) {
      return;
    }
    const importSpecifier = importDeclaration.getNamedImports().find((n) => n.getName() === importName2);
    if (importSpecifier) {
      importSpecifier.remove();
    }
    if (importDeclaration.getNamedImports().length === 0) {
      importDeclaration.remove();
    }
  };
  if (Array.isArray(importName)) {
    importName.forEach((name) => {
      removeImport(sourceFile, name, moduleSpecifier);
    });
  } else {
    removeImport(sourceFile, importName, moduleSpecifier);
  }
}

// src/angular/migrations/standalone/0002-import-standalone-component.ts
var migrateComponents = async (project, cliOptions) => {
  for (const sourceFile of project.getSourceFiles()) {
    if (sourceFile.getFilePath().endsWith(".html")) {
      const htmlAsString = sourceFile.getFullText();
      const {
        skippedIconsHtml,
        ionIcons,
        ionicComponents,
        hasRouterLink,
        hasRouterLinkWithHref
      } = detectIonicComponentsAndIcons(htmlAsString, sourceFile.getFilePath());
      if (ionicComponents.length > 0 || ionIcons.length > 0) {
        const tsSourceFile = findComponentTypescriptFileForTemplateFile(sourceFile);
        if (tsSourceFile) {
          await migrateAngularComponentClass(
            tsSourceFile,
            ionicComponents,
            ionIcons,
            skippedIconsHtml,
            hasRouterLink,
            hasRouterLinkWithHref,
            cliOptions
          );
          await saveFileChanges(tsSourceFile, cliOptions);
        }
      }
    } else if (sourceFile.getFilePath().endsWith(".ts")) {
      const templateAsString = getComponentTemplateAsString(sourceFile);
      if (templateAsString) {
        const {
          skippedIconsHtml,
          ionIcons,
          ionicComponents,
          hasRouterLink,
          hasRouterLinkWithHref
        } = detectIonicComponentsAndIcons(
          templateAsString,
          sourceFile.getFilePath()
        );
        await migrateAngularComponentClass(
          sourceFile,
          ionicComponents,
          ionIcons,
          skippedIconsHtml,
          hasRouterLink,
          hasRouterLinkWithHref,
          cliOptions
        );
        if (ionicComponents.length > 0 || ionIcons.length > 0) {
          await saveFileChanges(sourceFile, cliOptions);
        }
      }
    }
  }
};
async function migrateAngularComponentClass(sourceFile, ionicComponents, ionIcons, skippedIconsHtml, hasRouterLink, hasRouterLinkWithHref, cliOptions) {
  let ngModuleSourceFile;
  let modifiedNgModule = false;
  if (!isAngularComponentStandalone(sourceFile)) {
    ngModuleSourceFile = findNgModuleClassForComponent(sourceFile);
  }
  const hasIcons = ionIcons.length > 0;
  if (hasIcons) {
    addImportToClass(sourceFile, "addIcons", "ionicons");
    for (const ionIcon of ionIcons) {
      const iconName = kebabCaseToCamelCase(ionIcon);
      addImportToClass(sourceFile, iconName, "ionicons/icons");
    }
    insertAddIconsIntoConstructor(
      sourceFile,
      ionIcons.map((i) => kebabCaseToCamelCase(i))
    );
  }
  if (hasRouterLink) {
    addImportToClass(sourceFile, "IonRouterLink", "@ionic/angular/standalone");
    addImportToComponentDecorator(sourceFile, "IonRouterLink");
  }
  if (hasRouterLinkWithHref) {
    addImportToClass(
      sourceFile,
      "IonRouterLinkWithHref",
      "@ionic/angular/standalone"
    );
    addImportToComponentDecorator(sourceFile, "IonRouterLinkWithHref");
  }
  for (const ionicComponent of ionicComponents) {
    if (isAngularComponentStandalone(sourceFile)) {
      const componentClassName = kebabCaseToPascalCase(ionicComponent);
      addImportToComponentDecorator(sourceFile, componentClassName);
      removeImportFromComponentDecorator(sourceFile, "IonicModule");
      removeImportFromClass(sourceFile, "IonicModule", "@ionic/angular");
      addImportToClass(
        sourceFile,
        componentClassName,
        "@ionic/angular/standalone"
      );
      removeImportFromClass(sourceFile, componentClassName, "@ionic/angular");
    } else if (ngModuleSourceFile) {
      const componentClassName = kebabCaseToPascalCase(ionicComponent);
      addImportToClass(
        ngModuleSourceFile,
        componentClassName,
        "@ionic/angular/standalone"
      );
      addImportToNgModuleDecorator(ngModuleSourceFile, componentClassName);
      removeIonicModuleFromNgModule(ngModuleSourceFile);
      modifiedNgModule = true;
    }
  }
  if (skippedIconsHtml.length > 0) {
    log2.warning("--------------------------------------------------");
    log2.warning(
      `Dynamic ion-icon name detected in component template: ${sourceFile.getFilePath()}`
    );
    log2.warning(`Ionic is unable to automatically migrate these icons.`);
    log2.warning(
      `You will need to manually import these icons into your component:`
    );
    for (const skippedIcon of skippedIconsHtml) {
      log2.warning(`	${skippedIcon}`);
    }
    log2.warning("--------------------------------------------------");
  }
  if (modifiedNgModule && ngModuleSourceFile) {
    await saveFileChanges(ngModuleSourceFile, cliOptions);
  }
}
function removeIonicModuleFromNgModule(ngModuleSourceFile) {
  const ionicModuleImportDeclaration = ngModuleSourceFile.getImportDeclaration("@ionic/angular");
  const ionicModuleImportSpecifier = ionicModuleImportDeclaration == null ? void 0 : ionicModuleImportDeclaration.getNamedImports().find((n) => n.getName() === "IonicModule");
  if (ionicModuleImportSpecifier) {
    ionicModuleImportSpecifier.remove();
  }
  if ((ionicModuleImportDeclaration == null ? void 0 : ionicModuleImportDeclaration.getNamedImports().length) === 0) {
    ionicModuleImportDeclaration.remove();
  }
  const ngModuleDecorator = ngModuleSourceFile.getClasses()[0].getDecorator("NgModule");
  if (ngModuleDecorator) {
    deleteFromDecoratorArgArray(ngModuleDecorator, "imports", "IonicModule");
    deleteFromDecoratorArgArray(ngModuleDecorator, "exports", "IonicModule");
  }
}
function insertAddIconsIntoConstructor(sourceFile, icons) {
  const constructor = getOrCreateConstructor(sourceFile);
  constructor.addStatements(`addIcons({ ${icons.join(", ")} });`);
}
function detectIonicComponentsAndIcons(htmlAsString, filePath) {
  const ast = parse(htmlAsString, { filePath });
  const nodes = ast.templateNodes;
  const ionicComponents = [];
  const ionIcons = [];
  const skippedIconsHtml = [];
  let hasRouterLinkWithHref = false;
  let hasRouterLink = false;
  const recursivelyFindIonicComponents = (node) => {
    if (node.type === "Element$1" || node.type === "Template") {
      const tagName = node.type === "Template" ? node.tagName : node.name;
      if (IONIC_COMPONENTS.includes(tagName)) {
        if (!ionicComponents.includes(tagName)) {
          ionicComponents.push(tagName);
        }
        const routerLink = node.attributes.find(
          (a) => a.name === "routerLink" || a.name == "routerDirection" || a.name === "routerAction"
        ) !== void 0;
        if (!hasRouterLink && routerLink) {
          hasRouterLink = true;
        }
      }
      if (node.name === "ion-icon") {
        const staticNameAttribute = node.attributes.find(
          (a) => a.name === "name" || a.name === "icon"
        );
        if (staticNameAttribute) {
          const iconName = staticNameAttribute.value;
          if (!ionIcons.includes(iconName)) {
            ionIcons.push(iconName);
          }
        } else {
          const boundNameAttribute = node.inputs.find(
            (a) => a.name === "name" || a.name === "icon"
          );
          if (boundNameAttribute) {
            const skippedIcon = node.sourceSpan.toString();
            const iconNameRegex = /{{\s*'([^']+)'\s*}}/;
            const iconNameMatch = skippedIcon.match(iconNameRegex);
            if (iconNameMatch) {
              if (!ionIcons.includes(iconNameMatch[1])) {
                ionIcons.push(iconNameMatch[1]);
              }
            } else {
              skippedIconsHtml.push(skippedIcon);
            }
          }
        }
      }
      if (node.name === "a") {
        const routerLinkWithHref = node.attributes.find(
          (a) => a.name === "routerLink" || a.name == "routerDirection" || a.name === "routerAction"
        ) !== void 0;
        if (!hasRouterLinkWithHref && routerLinkWithHref) {
          hasRouterLinkWithHref = true;
        }
      }
      if (node.children.length > 0) {
        for (const childNode of node.children) {
          recursivelyFindIonicComponents(childNode);
        }
      }
    }
  };
  for (const node of nodes) {
    recursivelyFindIonicComponents(node);
  }
  return {
    ionicComponents,
    ionIcons,
    skippedIconsHtml,
    hasRouterLinkWithHref,
    hasRouterLink
  };
}
function getComponentTemplateAsString(sourceFile) {
  var _a, _b;
  if (isAngularComponentClass(sourceFile)) {
    const componentDecorator = getAngularComponentDecorator(sourceFile);
    const templatePropertyAssignment = getDecoratorArgument(
      componentDecorator,
      "template"
    );
    if (!templatePropertyAssignment) {
      return;
    }
    const templateLiteral = (_a = templatePropertyAssignment.getDescendantsOfKind(SyntaxKind4.NoSubstitutionTemplateLiteral)[0]) == null ? void 0 : _a.getLiteralValue();
    if (templateLiteral) {
      return templateLiteral;
    }
    const stringLiteral = (_b = templatePropertyAssignment.getDescendantsOfKind(SyntaxKind4.StringLiteral)[0]) == null ? void 0 : _b.getLiteralText();
    return stringLiteral;
  }
}

// src/angular/migrations/standalone/0003-migrate-bootstrap-application.ts
import { SyntaxKind as SyntaxKind5 } from "ts-morph";
var migrateBootstrapApplication = async (project, cliOptions) => {
  const sourceFile = project.getSourceFiles().find((sourceFile2) => sourceFile2.getFilePath().endsWith("src/main.ts"));
  if (sourceFile === void 0) {
    return;
  }
  const bootstrapApplicationCallExpression = sourceFile.getDescendantsOfKind(SyntaxKind5.CallExpression).find((callExpression) => {
    const identifier = callExpression.getFirstChildByKind(
      SyntaxKind5.Identifier
    );
    return identifier !== void 0 && identifier.getText() === "bootstrapApplication";
  });
  if (bootstrapApplicationCallExpression === void 0) {
    return;
  }
  const bootstrapApplicationCallExpressionArguments = bootstrapApplicationCallExpression.getArguments();
  if (bootstrapApplicationCallExpressionArguments.length === 0) {
    return;
  }
  const bootstrapApplicationOptionsArgument = bootstrapApplicationCallExpressionArguments[1];
  if (bootstrapApplicationOptionsArgument === void 0) {
    return;
  }
  if (bootstrapApplicationOptionsArgument.isKind(
    SyntaxKind5.ObjectLiteralExpression
  )) {
    const objectLiteralExpression = bootstrapApplicationOptionsArgument;
    const providersPropertyAssignment = objectLiteralExpression.getProperty("providers");
    if (providersPropertyAssignment === void 0) {
      return;
    }
    const providersArray = providersPropertyAssignment.getFirstChildByKind(
      SyntaxKind5.ArrayLiteralExpression
    );
    if (providersArray === void 0) {
      return;
    }
    const importProvidersFromFunctionCall = providersArray.getFirstChildByKind(
      SyntaxKind5.CallExpression
    );
    if (importProvidersFromFunctionCall === void 0) {
      return;
    }
    const importProvidersFromFunctionCallIdentifier = importProvidersFromFunctionCall.getFirstChildByKind(
      SyntaxKind5.Identifier
    );
    if (importProvidersFromFunctionCallIdentifier === void 0) {
      return;
    }
    if (importProvidersFromFunctionCallIdentifier.getText() !== "importProvidersFrom") {
      return;
    }
    const importProvidersFromFunctionCallArguments = importProvidersFromFunctionCall.getArguments();
    if (importProvidersFromFunctionCallArguments.length === 0) {
      return;
    }
    const importProvidersFromFunctionCallIonicModuleForRootCallExpression = importProvidersFromFunctionCallArguments.find((argument) => {
      return argument.getText().includes("IonicModule.forRoot");
    });
    if (importProvidersFromFunctionCallIonicModuleForRootCallExpression === void 0) {
      return;
    }
    if (importProvidersFromFunctionCallIonicModuleForRootCallExpression.isKind(
      SyntaxKind5.CallExpression
    )) {
      const importProvidersFromFunctionCallIonicModuleForRootCallExpressionArguments = importProvidersFromFunctionCallIonicModuleForRootCallExpression.getArguments();
      const ionicConfigObjectLiteralExpression = importProvidersFromFunctionCallIonicModuleForRootCallExpressionArguments[0];
      providersArray.addElement(
        `provideIonicAngular(${ionicConfigObjectLiteralExpression.getText()})`
      );
      importProvidersFromFunctionCall.removeArgument(
        importProvidersFromFunctionCallIonicModuleForRootCallExpression
      );
      if (importProvidersFromFunctionCall.getArguments().length === 0) {
        providersArray.removeElement(importProvidersFromFunctionCall);
      }
      providersArray.formatText();
      migrateIonicAngularImportDeclarations(sourceFile);
      return await saveFileChanges(sourceFile, cliOptions);
    }
  }
};
function migrateIonicAngularImportDeclarations(sourceFile) {
  const importDeclaration = sourceFile.getImportDeclaration("@ionic/angular");
  if (!importDeclaration) {
    return;
  }
  importDeclaration.setModuleSpecifier("@ionic/angular/standalone");
  const namedImports = importDeclaration.getNamedImports();
  const importSpecifier = namedImports.find(
    (n) => n.getName() === "IonicModule"
  );
  if (importSpecifier) {
    importSpecifier.remove();
  }
  importDeclaration.addNamedImport("provideIonicAngular");
}

// src/angular/migrations/standalone/0005-migrate-angular-json-assets.ts
var migrateAngularJsonAssets = async (project, cliOptions) => {
  const angularJsonSourceFile = project.getSourceFiles().find((sourceFile) => sourceFile.getFilePath().endsWith("angular.json"));
  if (angularJsonSourceFile === void 0) {
    return;
  }
  const angularJson = JSON.parse(angularJsonSourceFile.getText());
  for (const project2 of Object.keys(angularJson.projects)) {
    const assets = angularJson.projects[project2].architect.build.options.assets;
    const assetsToRemove = assets.filter((asset) => {
      return typeof asset === "object" && asset.input === "node_modules/ionicons/dist/ionicons/svg";
    });
    assetsToRemove.forEach((assetToRemove) => {
      const index = assets.indexOf(assetToRemove);
      assets.splice(index, 1);
    });
  }
  angularJsonSourceFile.replaceWithText(JSON.stringify(angularJson, null, 2));
  return await saveFileChanges(angularJsonSourceFile, cliOptions);
};

// src/angular/migrations/standalone/index.ts
import { group, confirm, log as log3 } from "@clack/prompts";

// src/angular/utils/package-utils.ts
import { existsSync } from "node:fs";
import { readFile } from "node:fs/promises";
var getActualPackageVersion = async (dir, packageName) => {
  const packageJsonPath = `${dir}/node_modules/${packageName}/package.json`;
  if (!existsSync(packageJsonPath)) {
    return null;
  }
  try {
    const packageJson = await readFile(packageJsonPath, { encoding: "utf-8" });
    const packageJsonContents = JSON.parse(packageJson);
    const version = packageJsonContents.version;
    return version;
  } catch (e) {
    return null;
  }
};

// src/angular/migrations/standalone/index.ts
var runStandaloneMigration = async ({
  project,
  cliOptions,
  dir,
  spinner: spinner3
}) => {
  const hasIonicAngularMinVersion = await checkInstalledIonicVersion(dir);
  if (!hasIonicAngularMinVersion) {
    return false;
  }
  spinner3.start(`Migrating project located at: ${dir}`);
  await migrateAppModule(project, cliOptions);
  await migrateBootstrapApplication(project, cliOptions);
  await migrateComponents(project, cliOptions);
  await migrateImportStatements(project, cliOptions);
  await migrateAngularJsonAssets(project, cliOptions);
  spinner3.stop(`Project migration at ${dir} completed successfully.`);
  log3.success(
    "We recommend reviewing the changes made by this migration and formatting your code (e.g., with Prettier) before committing."
  );
  return true;
};
async function checkInstalledIonicVersion(dir) {
  const ionicAngularVersion = await getActualPackageVersion(
    dir,
    "@ionic/angular"
  );
  if (!ionicAngularVersion) {
    log3.warn(
      "We could not detect the version of @ionic/angular installed in your project."
    );
    log3.warn(
      "This migration requires @ionic/angular version of 7.5.0 or later."
    );
    log3.warn("Do you want to proceed anyway?");
    const { continue: shouldContinue } = await group({
      continue: () => confirm({
        message: "Continue?",
        initialValue: false
      })
    });
    if (!shouldContinue || typeof shouldContinue !== "boolean") {
      log3.info("Migration canceled.");
      return false;
    }
  } else {
    const [major, minor] = ionicAngularVersion.split(".");
    const majorVersion = parseInt(major);
    const minorVersion = parseInt(minor);
    const logVersionError = () => {
      log3.error(
        "This migration requires an @ionic/angular version of v7.5.0 or greater."
      );
      log3.error("Install the latest version of @ionic/angular and try again.");
      log3.error("Migration canceled.");
    };
    if (majorVersion < 7) {
      logVersionError();
      return false;
    }
    if (minorVersion < 5) {
      logVersionError();
      return false;
    }
  }
  return true;
}

// src/index.ts
var IONIC_MIGRATION_GUIDE_URL = "https://www.ionicframework.com/docs/angular/build-options#migrating-from-modules-to-standalone";
var IONIC_REPOSITORY_ISSUES_URL = "https://github.com/ionic-team/ionic-angular-standalone-codemods/issues";
var isInteractive = () => TERMINAL_INFO.tty && !TERMINAL_INFO.ci && !process.argv.includes("--non-interactive");
async function main() {
  console.clear();
  intro("Ionic Angular Standalone Codemods");
  intro(
    "This utility will migrate your Ionic Angular project to use the new standalone components from Ionic Framework v7.5.0."
  );
  log4.warning("--------------------------------------------------");
  log4.warning(
    "\u26A0\uFE0F  This utility is experimental. Always review the changes made before committing them to your project. \u26A0\uFE0F"
  );
  log4.warning(
    `For manual migration, see the guide at: ${color.underline(
      IONIC_MIGRATION_GUIDE_URL
    )}`
  );
  log4.warning("--------------------------------------------------");
  const cli = isInteractive() ? await group2({
    dryRun: () => confirm2({
      message: "Would you like to run this migration as a dry run? No changes will be written to your project.",
      initialValue: true
    }),
    dir: () => text({
      message: "Please enter the path to your project (default is the current working directory):",
      initialValue: cwd()
    })
  }) : {
    // If we are in a non-interactive terminal then use defaults
    dryRun: false,
    dir: cwd()
  };
  if (typeof cli.dryRun !== "boolean") {
    return;
  }
  let project;
  if (existsSync2(`${cli.dir}/tsconfig.json`)) {
    project = new Project3({
      tsConfigFilePath: `${cli.dir}/tsconfig.json`
    });
  } else {
    project = new Project3();
  }
  const s = spinner2();
  project.addSourceFilesAtPaths([
    `${cli.dir}/src/**/*.html`,
    `${cli.dir}/src/**/*.ts`,
    `${cli.dir}/angular.json`
  ]);
  try {
    await runStandaloneMigration({
      project,
      cliOptions: cli,
      dir: cli.dir,
      spinner: s
    });
  } catch (e) {
    s.stop("An error occurred during the migration.", 1);
    log4.error(e.message);
  }
  outro(
    `If you encounter any issues with this migration utility, please report them at: ${color.underline(
      IONIC_REPOSITORY_ISSUES_URL
    )}`
  );
}
main().catch(console.error);
